// Ghost.java
// Used for PacMan

import ch.aplu.jgamegrid.*;
import java.awt.Color;

public class Ghost extends Actor
{
  private PacActor pacActor;
  private int type;

  public Ghost(PacActor pacActor, int type)
  {
    super("sprites/ghost_" + type + ".gif");
    this.pacActor = pacActor;
    this.type = type;
  }

  public void act()
  {
    moveStrategy();
    if (getDirection() > 150 && getDirection() < 210)
      setHorzMirror(false);
    else
      setHorzMirror(true);
  }

  private void moveStrategy()
  {
    Location pacLocation = pacActor.getLocation();
    double oldDirection = getDirection();

    // Use an approach strategy
    Location.CompassDirection compassDir =
      getLocation().get4CompassDirectionTo(pacLocation);
    Location best = getLocation().getNeighbourLocation(compassDir);
    setDirection(compassDir);
    if (type == 0 && canMove(best))
    {
      move();
    }
    else // normal movement
    {
      setDirection(oldDirection);

      Location next = getNextMoveLocation();
      if (canMove(next)) // Move forward
        setLocation(next);
      else  // Move left or right at random
      {
        int sign;
        if (Math.random() < 0.5)
          sign = 1;
        else
          sign = -1;
        turn(sign*90);
        next = getNextMoveLocation();
        if (canMove(next))
          setLocation(next);
        else
        {
          turn(180);
          next = getNextMoveLocation();
          if (canMove(next))
            setLocation(next);
          else
          {
            turn(-sign*90); // Backward
            next = getNextMoveLocation();
            if (canMove(next))
              setLocation(next);
          }
        }
      }
    }
  }

  private boolean canMove(Location location)
  {
    Color c = getBackground().getColor(location);
    if (c.equals(Color.gray))
      return false;
    else
      return true;
  }
}
